# Generated by roxygen2: do not edit by hand

S3method("[",ggeffects)
S3method(as.data.frame,ggeffects)
S3method(format,ggcomparisons)
S3method(format,ggeffects)
S3method(get_predictions,Gam)
S3method(get_predictions,MCMCglmm)
S3method(get_predictions,MixMod)
S3method(get_predictions,averaging)
S3method(get_predictions,bamlss)
S3method(get_predictions,bayesglm)
S3method(get_predictions,bayesx)
S3method(get_predictions,betareg)
S3method(get_predictions,bigglm)
S3method(get_predictions,biglm)
S3method(get_predictions,bracl)
S3method(get_predictions,brglm)
S3method(get_predictions,brmsfit)
S3method(get_predictions,brmultinom)
S3method(get_predictions,cgam)
S3method(get_predictions,clm)
S3method(get_predictions,clm2)
S3method(get_predictions,clmm)
S3method(get_predictions,coxph)
S3method(get_predictions,default)
S3method(get_predictions,feglm)
S3method(get_predictions,fixest)
S3method(get_predictions,flac)
S3method(get_predictions,flic)
S3method(get_predictions,gam)
S3method(get_predictions,gamlss)
S3method(get_predictions,gee)
S3method(get_predictions,geeglm)
S3method(get_predictions,glimML)
S3method(get_predictions,glm)
S3method(get_predictions,glmRob)
S3method(get_predictions,glmerMod)
S3method(get_predictions,glmgee)
S3method(get_predictions,glmmPQL)
S3method(get_predictions,glmmTMB)
S3method(get_predictions,glmrob)
S3method(get_predictions,glmx)
S3method(get_predictions,gls)
S3method(get_predictions,hurdle)
S3method(get_predictions,ivreg)
S3method(get_predictions,lm)
S3method(get_predictions,lmRob)
S3method(get_predictions,lm_robust)
S3method(get_predictions,lme)
S3method(get_predictions,lmerMod)
S3method(get_predictions,lmrob)
S3method(get_predictions,logistf)
S3method(get_predictions,logitr)
S3method(get_predictions,lrm)
S3method(get_predictions,mblogit)
S3method(get_predictions,mclogit)
S3method(get_predictions,merMod)
S3method(get_predictions,merModLmerTest)
S3method(get_predictions,mixor)
S3method(get_predictions,mlogit)
S3method(get_predictions,multinom)
S3method(get_predictions,multinom_weightit)
S3method(get_predictions,negbin)
S3method(get_predictions,nestedLogit)
S3method(get_predictions,nlmerMod)
S3method(get_predictions,ols)
S3method(get_predictions,ordinal_weightit)
S3method(get_predictions,orm)
S3method(get_predictions,phyloglm)
S3method(get_predictions,phylolm)
S3method(get_predictions,plm)
S3method(get_predictions,polr)
S3method(get_predictions,rlmerMod)
S3method(get_predictions,rq)
S3method(get_predictions,rqs)
S3method(get_predictions,rqss)
S3method(get_predictions,sdmTMB)
S3method(get_predictions,speedglm)
S3method(get_predictions,speedlm)
S3method(get_predictions,stanreg)
S3method(get_predictions,survreg)
S3method(get_predictions,svyglm)
S3method(get_predictions,svyglm.nb)
S3method(get_predictions,tobit)
S3method(get_predictions,truncreg)
S3method(get_predictions,vgam)
S3method(get_predictions,vglm)
S3method(get_predictions,wblm)
S3method(get_predictions,wbm)
S3method(get_predictions,zeroinfl)
S3method(get_predictions,zerotrunc)
S3method(plot,ggalleffects)
S3method(plot,ggeffects)
S3method(plot,see_equivalence_test_ggeffects)
S3method(print,ggcomparisons)
S3method(print,ggeffects)
S3method(print_html,ggcomparisons)
S3method(print_html,ggeffects)
S3method(print_md,ggcomparisons)
S3method(print_md,ggeffects)
S3method(residualize_over_grid,data.frame)
S3method(residualize_over_grid,ggeffects)
S3method(test_predictions,default)
S3method(test_predictions,ggeffects)
S3method(vcov,ggeffects)
export(collapse_by_group)
export(data_grid)
export(get_complete_df)
export(get_legend_labels)
export(get_legend_title)
export(get_predictions)
export(get_title)
export(get_x_labels)
export(get_x_title)
export(get_y_title)
export(ggaverage)
export(ggeffect)
export(ggeffects_palette)
export(ggemmeans)
export(ggpredict)
export(hypothesis_test)
export(install_latest)
export(new_data)
export(pool_comparisons)
export(pool_predictions)
export(predict_response)
export(pretty_range)
export(print_html)
export(print_md)
export(representative_values)
export(residualize_over_grid)
export(show_palettes)
export(test_predictions)
export(theme_ggeffects)
export(values_at)
importFrom(insight,print_html)
importFrom(insight,print_md)
