#' @docType data
#' @title Sample dataset from the EUROFAMCARE project
#' @name efc
#' @aliases efc_test
#' @keywords data
#'
#' @description An SPSS sample data set, imported with the [`sjlabelled::read_spss()`]
#' function. Consists of 28 variables from 908 observations. The data set is part
#' of the EUROFAMCARE project, a cross-national survey on informal caregiving in
#' Europe.
#'
#' @examples
#' # Attach EFC-data
#' data(efc)
#'
#' # Show structure
#' str(efc)
#'
#' # show first rows
#' head(efc)
NULL
